import json

from knowledge_base import KnowledgeBase

examples = [
    {
        "text": "icecream is cold",
        "parsed": ("exists-property-with-value", 'icecream', 'cold'),
    },
    {
        "text": "lava is dangerous",
        "parsed": ("exists-property-with-value", 'lava', 'dangerous')
    },
    {
        "text": "earth is a planet",
        "parsed": ("pertenence-to-group", 'earth', 'planet'),
    },
    {
        "text": "Green is a color",
        "parsed": ("pertenence-to-group", 'green', 'color'),
    },
    {
        "text": "a plane can fly",
        "parsed": ("has-capacity", 'plane', 'fly')
    },
    {
        "text": "a wale can swim",
        "parsed": ("has-capacity", 'wale', 'swim')
    }
]

base_knowledge = {
    'icecream': {
        "groups": set(['noun', 'object', 'comestible', 'sweet']),
    },
    'lava': {
        "groups": set(['noun', 'object']),
    },
    'earth': {
        "groups": set(['noun', 'object', 'planet']),
    },
    'green': {
        "groups": set(['noun', 'color', 'concept']),
    },
    'plane': {
        "groups": set(['noun', 'object', 'vehicle', 'fast']),
    },
    'car': {
        "groups": set(['noun', 'object', 'vehicle', 'slow-ish']),
    },
    'wale': {
        "groups": set(['noun', 'object', 'living-being']),
    },
    'cold': {
        "groups": set(['property', 'temperature']),
        "as_property": "temperature",
    },
    'dangerous': {
        "groups": set(['property']),
        "as_property": "safety",
    },
    'planet': {
        "groups": set(['noun', 'group']),
    },
    'color': {
        "groups": set(['property', 'group']),
    },
    'fly': {
        "groups": set(['verb']),
    },
    'swim': {
        "groups": set(['verb']),
    },
}


def main():
    knowledge = KnowledgeBase(
        knowledge=base_knowledge,
    )

    differences = knowledge.train(examples)
    print("----")
    print(differences())
    print("----")

    for test in [{'text': 'a bus can run'}, {'text': 'io is a moon'}]:
        row = test['text'].lower().split()
        result, differences = knowledge.process(row)

        print("result:", result)
        print(differences())
        print()
    print('-----')
    print(json.dumps(sorted(knowledge.knowledge.keys()), indent=4))


if __name__ == '__main__':
    main()
